package de.meinefirma.meinprojekt.dao;

import java.util.List;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.transaction.UserTransaction;

/** Super-Klasse fuer alle DAOs mit einigen generellen CRUD-Methoden */
public abstract class AbstractDao
{
   protected final EntityManager   em;
   protected final UserTransaction tx;

   // Konstruktor mit Lookups:
   public AbstractDao()
   {
      try {
         em = (EntityManager)   (new InitialContext()).lookup( "java:comp/env/persistence/em" );
         tx = (UserTransaction) (new InitialContext()).lookup( "java:comp/UserTransaction" );
      } catch( NamingException ex ) {
         throw new RuntimeException( ex );
      }
   }

   // Erstelle neuen Eintrag:
   public <T> void createEntity( T entity )
   {
      try {
         tx.begin();
         em.persist( entity );
         tx.commit();
      } catch( Exception ex ) {
         try { tx.rollback(); } catch( Exception e ) { /* ok */ }
         throw new RuntimeException( ex );
      }
   }

   // Lies bestimmte Eintraege zu einer Entity-Klasse:
   @SuppressWarnings("unchecked")
   public <T> List<T> queryEntities( String namedQuery, String paramName, Object paramValue )
   {
      Query query = em.createNamedQuery( namedQuery );
      query.setParameter( paramName, paramValue );
      return query.getResultList();
   }

   // Lies bestimmte Eintraege zu einer Entity-Klasse:
   @SuppressWarnings("unchecked")
   public <T> List<T> queryEntities( String namedQuery, List<Object> paramNamesAndValues )
   {
      Query query = em.createNamedQuery( namedQuery );
      for( int i = 0; paramNamesAndValues != null && i < paramNamesAndValues.size(); i += 2 ) {
         query.setParameter( paramNamesAndValues.get( i ).toString(), paramNamesAndValues.get( i + 1 ) );
      }
      return query.getResultList();
   }

   // Native Query:
   @SuppressWarnings("unchecked")
   public List<Object[]> queryNative( String sqlCmd, List<Object> paramValues )
   {
      Query query = em.createNativeQuery( sqlCmd );
      for( int i = 0; paramValues != null && i < paramValues.size(); i++ ) {
         query.setParameter( i + 1, paramValues.get( i ) );
      }
      return query.getResultList();
   }

   // Lies alle Eintraege zu einer Entity-Klasse:
   @SuppressWarnings("unchecked")
   public <T> List<T> readAllEntities( Class<T> clss )
   {
      return em.createQuery( "Select d from " + clss.getSimpleName() + " d" ).getResultList();
   }

   // Loesche alle Eintraege zu einer Entity-Klasse:
   public <T> int deleteAllEntities( Class<T> clss )
   {
      try {
         tx.begin();
         em.flush();
         em.clear();
         int r = em.createQuery( "Delete from " + clss.getSimpleName() ).executeUpdate();
         tx.commit();
         return r;
      } catch( Exception ex ) {
         try { tx.rollback(); } catch( Exception e ) { /* ok */ }
         throw new RuntimeException( ex );
      }
   }
}
